@@ -1,5 +1,5 @@
 """AMQP Messaging Framework for Python"""
-VERSION = (1, 0, 7)
+VERSION = (1, 1, 0)
 __version__ = ".".join(map(str, VERSION[0:3])) + "".join(VERSION[3:])
 __author__ = "Ask Solem"
 __contact__ = "ask@celeryproject.org"
@@ -59,6 +59,6 @@ if not os.environ.get("KOMBU_NO_EVAL", False):
         "VERSION": VERSION})
 
 if os.environ.get("KOMBU_LOG_DEBUG"):
-    os.environ["KOMBU_LOG_CHANNEL"] = "1"
+    os.environ.update(KOMBU_LOG_CHANNEL="1", KOMBU_LOG_CONNECTION="1")
     from kombu.utils import debug
     debug.setup_logging()
