@@ -256,7 +256,7 @@ class Consumer(object):
         self.consume()
         return self
 
-    def __exit__(self):
+    def __exit__(self, *exc_info):
         self.cancel()
 
     def consume(self, no_ack=None):
