@@ -1,5 +1,5 @@
 import pickle
-import unittest2 as unittest
+from kombu.tests.utils import unittest
 
 from kombu.connection import BrokerConnection, Resource
 
@@ -97,6 +97,22 @@ class test_Connection(unittest.TestCase):
         self.assertTupleEqual(conn.connection_errors, (KeyError, ValueError))
 
 
+class test_Connection_With_Broker_Args(unittest.TestCase):
+
+    _extra_args = {
+        'pool_recycle': 3600,
+        'echo': True
+    }
+
+    def setUp(self):
+        self.conn = BrokerConnection(port=5672, transport=Transport,
+                                     transport_options=self._extra_args)
+
+    def test_establish_connection(self):
+        conn = self.conn
+        self.assertEqual(conn.transport_options, self._extra_args)
+
+
 class ResourceCase(unittest.TestCase):
     abstract = True
 
